@@ -57,29 +57,6 @@ class ClientException(Exception):
         self.exit_code = exit_code
 
 
-class IncrementalStyle(enum.Enum):
-    SHALLOW = "shallow"
-    FINE_GRAINED = "fine_grained"
-
-    def __str__(self) -> str:
-        return self.value
-
-
-class ProfileOutput(enum.Enum):
-    TRACE_EVENT: str = "trace_event"
-    COLD_START_PHASES: str = "cold_start_phases"
-    INCREMENTAL_UPDATES: str = "incremental_updates"
-    TAINT: str = "taint"
-    INDIVIDUAL_TABLE_SIZES: str = "individual_table_sizes"
-    TOTAL_SHARED_MEMORY_SIZE_OVER_TIME: str = "total_shared_memory_size_over_time"
-    TOTAL_SHARED_MEMORY_SIZE_OVER_TIME_GRAPH: str = (
-        "total_shared_memory_size_over_time_graph"  # noqa B950
-    )
-
-    def __str__(self) -> str:
-        return self.value
-
-
 class Result:
     def __init__(self, code: int, output: str, error: Optional[str] = None) -> None:
         self.code: int = code
