@@ -133,6 +133,7 @@ async def try_initialize(
     - If an exit notification is received, return `InitializationExit`. The LSP
       spec allows exiting a server without a preceding initialize request.
     """
+    request = None
     try:
         request = await lsp.read_json_rpc(input_channel)
         LOG.debug(f"Received pre-initialization LSP request: {request}")
@@ -182,7 +183,7 @@ async def try_initialize(
         await lsp.write_json_rpc(
             output_channel,
             json_rpc.ErrorResponse(
-                id=request.id,
+                id=request.id if request is not None else None,
                 code=json_rpc_error.error_code(),
                 message=str(json_rpc_error),
                 data={"retry": False},
@@ -195,6 +196,7 @@ async def try_initialize(
 async def _read_lsp_request(
     input_channel: connection.TextReader, output_channel: connection.TextWriter
 ) -> AsyncIterator[json_rpc.Request]:
+    message = None
     try:
         message = await lsp.read_json_rpc(input_channel)
         yield message
@@ -202,7 +204,8 @@ async def _read_lsp_request(
         await lsp.write_json_rpc(
             output_channel,
             json_rpc.ErrorResponse(
-                id=message.id,
+                # pyre-ignore[16] - refinement doesn't work here for some reason
+                id=message.id if message is not None else None,
                 code=json_rpc_error.error_code(),
                 message=str(json_rpc_error),
             ),
@@ -774,8 +777,10 @@ class PyreServerHandler(connection.BackgroundTask):
     async def _run(self, server_start_options: PyreServerStartOptions) -> None:
         server_identifier = server_start_options.server_identifier
         start_arguments = server_start_options.start_arguments
+        local_root = start_arguments.base_arguments.relative_local_root
         socket_path = server_connection.get_default_socket_path(
-            log_directory=Path(start_arguments.base_arguments.log_path)
+            project_root=Path(start_arguments.base_arguments.global_root),
+            relative_local_root=Path(local_root) if local_root else None,
         )
         try:
             async with connection.connect_in_text_mode(socket_path) as (
