@@ -451,7 +451,7 @@ class PartialConfiguration:
             ignore_infer=[],
             isolation_prefix=arguments.isolation_prefix,
             logger=arguments.logger,
-            number_of_workers=None,
+            number_of_workers=arguments.number_of_workers,
             oncall=None,
             other_critical_files=[],
             python_version=(
@@ -1200,7 +1200,7 @@ class Configuration:
                     f"`{extension.suffix}`"
                 )
             else:
-                vaild_extensions.append(extension.suffix)
+                vaild_extensions.append(extension.command_line_argument())
         return vaild_extensions
 
     def get_isolation_prefix_respecting_override(self) -> Optional[str]:
