@@ -153,83 +153,59 @@ def run_pyre_command(
 
 def _run_check_command(arguments: command_arguments.CommandArguments) -> ExitCode:
     configuration = _create_configuration_with_retry(arguments, Path("."))
-    if configuration.use_command_v2:
-        _check_configuration(configuration)
-        _start_logging_to_directory(configuration.log_directory)
-        check_arguments = command_arguments.CheckArguments(
-            debug=arguments.debug,
-            enable_memory_profiling=arguments.enable_memory_profiling,
-            enable_profiling=arguments.enable_profiling,
-            log_identifier=arguments.log_identifier,
-            logging_sections=arguments.logging_sections,
-            noninteractive=arguments.noninteractive,
-            output=arguments.output,
-            sequential=arguments.sequential,
-            show_error_traces=arguments.show_error_traces,
-        )
-        return v2.check.run(configuration, check_arguments)
-    else:
-        return run_pyre_command(
-            commands.Check(
-                arguments, original_directory=os.getcwd(), configuration=configuration
-            ),
-            configuration,
-            arguments.noninteractive,
-        )
+    _check_configuration(configuration)
+    _start_logging_to_directory(configuration.log_directory)
+    check_arguments = command_arguments.CheckArguments(
+        debug=arguments.debug,
+        enable_memory_profiling=arguments.enable_memory_profiling,
+        enable_profiling=arguments.enable_profiling,
+        log_identifier=arguments.log_identifier,
+        logging_sections=arguments.logging_sections,
+        noninteractive=arguments.noninteractive,
+        output=arguments.output,
+        sequential=arguments.sequential,
+        show_error_traces=arguments.show_error_traces,
+    )
+    return v2.check.run(configuration, check_arguments)
 
 
 def _run_incremental_command(
     arguments: command_arguments.CommandArguments,
     nonblocking: bool,
-    incremental_style: commands.IncrementalStyle,
+    incremental_style: command_arguments.IncrementalStyle,
     no_start_server: bool,
     no_watchman: bool,
 ) -> ExitCode:
     configuration = _create_configuration_with_retry(arguments, Path("."))
-    if configuration.use_command_v2:
-        _check_configuration(configuration)
-        _start_logging_to_directory(configuration.log_directory)
-        start_arguments = command_arguments.StartArguments(
-            changed_files_path=arguments.changed_files_path,
-            debug=arguments.debug,
-            enable_memory_profiling=arguments.enable_memory_profiling,
-            enable_profiling=arguments.enable_profiling,
-            load_initial_state_from=arguments.load_initial_state_from,
-            log_identifier=arguments.log_identifier,
-            logging_sections=arguments.logging_sections,
-            no_saved_state=arguments.no_saved_state,
-            no_watchman=no_watchman,
-            noninteractive=arguments.noninteractive,
-            save_initial_state_to=arguments.save_initial_state_to,
-            saved_state_project=arguments.saved_state_project,
-            sequential=arguments.sequential,
-            show_error_traces=arguments.show_error_traces,
-            store_type_check_resolution=False,
-            terminal=False,
-            wait_on_initialization=True,
-        )
-        return v2.incremental.run(
-            configuration,
-            command_arguments.IncrementalArguments(
-                output=arguments.output,
-                no_start=no_start_server,
-                start_arguments=start_arguments,
-            ),
-        )
-    else:
-        return run_pyre_command(
-            commands.Incremental(
-                arguments,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                nonblocking=nonblocking,
-                incremental_style=incremental_style,
-                no_start_server=no_start_server,
-                no_watchman=no_watchman,
-            ),
-            configuration,
-            arguments.noninteractive,
-        )
+    _check_configuration(configuration)
+    _start_logging_to_directory(configuration.log_directory)
+    start_arguments = command_arguments.StartArguments(
+        changed_files_path=arguments.changed_files_path,
+        debug=arguments.debug,
+        enable_memory_profiling=arguments.enable_memory_profiling,
+        enable_profiling=arguments.enable_profiling,
+        load_initial_state_from=arguments.load_initial_state_from,
+        log_identifier=arguments.log_identifier,
+        logging_sections=arguments.logging_sections,
+        no_saved_state=arguments.no_saved_state,
+        no_watchman=no_watchman,
+        noninteractive=arguments.noninteractive,
+        save_initial_state_to=arguments.save_initial_state_to,
+        saved_state_project=arguments.saved_state_project,
+        sequential=arguments.sequential,
+        show_error_traces=arguments.show_error_traces,
+        store_type_check_resolution=False,
+        terminal=False,
+        wait_on_initialization=True,
+    )
+    return v2.incremental.run(
+        configuration,
+        command_arguments.IncrementalArguments(
+            output=arguments.output,
+            no_start=no_start_server,
+            start_arguments=start_arguments,
+        ),
+    )
 
 
 def _run_default_command(arguments: command_arguments.CommandArguments) -> ExitCode:
@@ -237,7 +213,7 @@ def _run_default_command(arguments: command_arguments.CommandArguments) -> ExitC
         return _run_incremental_command(
             arguments=arguments,
             nonblocking=False,
-            incremental_style=commands.IncrementalStyle.FINE_GRAINED,
+            incremental_style=command_arguments.IncrementalStyle.FINE_GRAINED,
             no_start_server=False,
             no_watchman=False,
         )
@@ -252,25 +228,6 @@ def _run_default_command(arguments: command_arguments.CommandArguments) -> ExitC
         return _run_check_command(arguments)
 
 
-def _run_servers_list_command(
-    arguments: command_arguments.CommandArguments,
-) -> ExitCode:
-    if arguments.use_command_v2 is not False:
-        return v2.servers.run_list(arguments.output)
-    else:
-        configuration = configuration_module.create_configuration(arguments, Path("."))
-        return run_pyre_command(
-            commands.Servers(
-                arguments,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                subcommand="list",
-            ),
-            configuration,
-            arguments.noninteractive,
-        )
-
-
 def _create_configuration_with_retry(
     arguments: command_arguments.CommandArguments, base_directory: Path
 ) -> configuration_module.Configuration:
@@ -361,7 +318,8 @@ def _check_configuration(configuration: configuration_module.Configuration) -> N
 @click.option(
     "--output",
     type=click.Choice(
-        [command_arguments.TEXT, command_arguments.JSON], case_sensitive=False
+        [command_arguments.TEXT, command_arguments.JSON, command_arguments.SARIF],
+        case_sensitive=False,
     ),
     default=command_arguments.TEXT,
     help="How to format output.",
@@ -478,6 +436,7 @@ def _check_configuration(configuration: configuration_module.Configuration) -> N
     help="Power of the hash table in shared memory.",
     hidden=True,
 )
+@click.option("--number-of-workers", type=int, help="Number of parallel workers to use")
 def pyre(
     context: click.Context,
     local_configuration: Optional[str],
@@ -518,6 +477,7 @@ def pyre(
     shared_memory_heap_size: Optional[int],
     shared_memory_dependency_table_power: Optional[int],
     shared_memory_hash_table_power: Optional[int],
+    number_of_workers: Optional[int],
 ) -> int:
     arguments = command_arguments.CommandArguments(
         local_configuration=local_configuration,
@@ -560,6 +520,7 @@ def pyre(
         shared_memory_heap_size=shared_memory_heap_size,
         shared_memory_dependency_table_power=shared_memory_dependency_table_power,
         shared_memory_hash_table_power=shared_memory_hash_table_power,
+        number_of_workers=number_of_workers,
     )
     if arguments.version:
         _show_pyre_version(arguments)
@@ -663,8 +624,7 @@ def analyze(
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
 
-    # TODO: Change the condition to `configuration.use_command_v2` when ready
-    if False:
+    if configuration.use_command_v2:
         _check_configuration(configuration)
         _start_logging_to_directory(configuration.log_directory)
         return v2.analyze.run(
@@ -747,11 +707,11 @@ def check(context: click.Context) -> int:
     "--incremental-style",
     type=click.Choice(
         [
-            str(commands.IncrementalStyle.SHALLOW),
-            str(commands.IncrementalStyle.FINE_GRAINED),
+            str(command_arguments.IncrementalStyle.SHALLOW),
+            str(command_arguments.IncrementalStyle.FINE_GRAINED),
         ]
     ),
-    default=str(commands.IncrementalStyle.FINE_GRAINED),
+    default=str(command_arguments.IncrementalStyle.FINE_GRAINED),
     help="[DEPRECATED] How to approach doing incremental checks.",
 )
 @click.option("--no-start", is_flag=True, default=False, hidden=True)
@@ -778,9 +738,9 @@ def incremental(
     return _run_incremental_command(
         arguments=command_argument,
         nonblocking=nonblocking,
-        incremental_style=commands.IncrementalStyle.SHALLOW
-        if incremental_style == str(commands.IncrementalStyle.SHALLOW)
-        else commands.IncrementalStyle.FINE_GRAINED,
+        incremental_style=command_arguments.IncrementalStyle.SHALLOW
+        if incremental_style == str(command_arguments.IncrementalStyle.SHALLOW)
+        else command_arguments.IncrementalStyle.FINE_GRAINED,
         no_start_server=no_start,
         no_watchman=no_watchman,
     )
@@ -968,25 +928,12 @@ def kill(context: click.Context, with_fire: bool) -> int:
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-    if configuration.use_command_v2:
-        return v2.kill.run(configuration, with_fire)
-    else:
-        return run_pyre_command(
-            commands.Kill(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                with_fire=with_fire,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    return v2.kill.run(configuration, with_fire)
 
 
 @pyre.command()
-@click.option("--no-watchman", is_flag=True, default=False, hidden=True)
 @click.pass_context
-def persistent(context: click.Context, no_watchman: bool) -> int:
+def persistent(context: click.Context) -> int:
     """
     Entry point for IDE integration to Pyre. Communicates with a Pyre server using
     the Language Server Protocol, accepts input from stdin and writing diagnostics
@@ -997,30 +944,16 @@ def persistent(context: click.Context, no_watchman: bool) -> int:
     configuration = configuration_module.create_configuration(
         command_argument, base_directory
     )
-    if configuration.use_command_v2:
-        _start_logging_to_directory(
-            configuration.log_directory,
-        )
-        # NOTE(grievejia): `--no-watchman` has no effect here. We might want to
-        # either respect it or deprecate it at some point.
-        return v2.persistent.run(
-            command_argument,
-            base_directory,
-            v2.backend_arguments.RemoteLogging.create(
-                configuration.logger, command_argument.log_identifier
-            ),
-        )
-    else:
-        return run_pyre_command(
-            commands.Persistent(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                no_watchman=no_watchman,
-            ),
-            configuration,
-            True,
-        )
+    _start_logging_to_directory(
+        configuration.log_directory,
+    )
+    return v2.persistent.run(
+        command_argument,
+        base_directory,
+        v2.backend_arguments.RemoteLogging.create(
+            configuration.logger, command_argument.log_identifier
+        ),
+    )
 
 
 @pyre.command()
@@ -1066,8 +999,8 @@ def pysa_language_server(context: click.Context, no_watchman: bool) -> int:
 @pyre.command()
 @click.option(
     "--profile-output",
-    type=click.Choice([str(x) for x in commands.ProfileOutput]),
-    default=str(commands.ProfileOutput.COLD_START_PHASES),
+    type=click.Choice([str(x) for x in command_arguments.ProfileOutput]),
+    default=str(command_arguments.ProfileOutput.COLD_START_PHASES),
     help="Specify what to output.",
 )
 @click.pass_context
@@ -1076,27 +1009,15 @@ def profile(context: click.Context, profile_output: str) -> int:
     Display profiling output.
     """
 
-    def get_profile_output(profile_output: str) -> commands.ProfileOutput:
-        for item in commands.ProfileOutput:
+    def get_profile_output(profile_output: str) -> command_arguments.ProfileOutput:
+        for item in command_arguments.ProfileOutput:
             if str(item) == profile_output:
                 return item
         raise ValueError(f"Unrecognized value for --profile-output: {profile_output}")
 
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    if configuration.use_command_v2:
-        return v2.profile.run(configuration, get_profile_output(profile_output))
-    else:
-        return run_pyre_command(
-            commands.Profile(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                profile_output=get_profile_output(profile_output),
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    return v2.profile.run(configuration, get_profile_output(profile_output))
 
 
 @pyre.command()
@@ -1113,21 +1034,8 @@ def query(context: click.Context, query: str) -> int:
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-
-    if configuration.use_command_v2:
-        _start_logging_to_directory(configuration.log_directory)
-        return v2.query.run(configuration, query)
-    else:
-        return run_pyre_command(
-            commands.Query(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                query=query,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    _start_logging_to_directory(configuration.log_directory)
+    return v2.query.run(configuration, query)
 
 
 @pyre.command()
@@ -1156,26 +1064,13 @@ def rage(
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-
-    if configuration.use_command_v2:
-        return v2.rage.run(
-            configuration,
-            command_arguments.RageArguments(
-                output=Path(output_file) if output_file is not None else None,
-                server_log_count=server_log_count,
-            ),
-        )
-    else:
-        return run_pyre_command(
-            commands.Rage(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                output_path=output_file,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    return v2.rage.run(
+        configuration,
+        command_arguments.RageArguments(
+            output=Path(output_file) if output_file is not None else None,
+            server_log_count=server_log_count,
+        ),
+    )
 
 
 @pyre.command()
@@ -1198,11 +1093,11 @@ def rage(
     "--incremental-style",
     type=click.Choice(
         [
-            str(commands.IncrementalStyle.SHALLOW),
-            str(commands.IncrementalStyle.FINE_GRAINED),
+            str(command_arguments.IncrementalStyle.SHALLOW),
+            str(command_arguments.IncrementalStyle.FINE_GRAINED),
         ]
     ),
-    default=str(commands.IncrementalStyle.FINE_GRAINED),
+    default=str(command_arguments.IncrementalStyle.FINE_GRAINED),
     help="[DEPRECATED] How to approach doing incremental checks.",
 )
 @click.pass_context
@@ -1218,52 +1113,35 @@ def restart(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    if configuration.use_command_v2:
-        _check_configuration(configuration)
-        _start_logging_to_directory(configuration.log_directory)
-        start_arguments = command_arguments.StartArguments(
-            changed_files_path=command_argument.changed_files_path,
-            debug=command_argument.debug,
-            enable_memory_profiling=command_argument.enable_memory_profiling,
-            enable_profiling=command_argument.enable_profiling,
-            load_initial_state_from=command_argument.load_initial_state_from,
-            log_identifier=command_argument.log_identifier,
-            logging_sections=command_argument.logging_sections,
-            no_saved_state=command_argument.no_saved_state,
-            no_watchman=no_watchman,
-            noninteractive=command_argument.noninteractive,
-            save_initial_state_to=command_argument.save_initial_state_to,
-            saved_state_project=command_argument.saved_state_project,
-            sequential=command_argument.sequential,
-            show_error_traces=command_argument.show_error_traces,
-            store_type_check_resolution=store_type_check_resolution,
-            terminal=terminal,
-            wait_on_initialization=True,
-        )
-        return v2.restart.run(
-            configuration,
-            command_arguments.IncrementalArguments(
-                output=command_argument.output,
-                no_start=False,
-                start_arguments=start_arguments,
-            ),
-        )
-    else:
-        return run_pyre_command(
-            commands.Restart(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                terminal=terminal,
-                store_type_check_resolution=store_type_check_resolution,
-                use_watchman=not no_watchman,
-                incremental_style=commands.IncrementalStyle.SHALLOW
-                if incremental_style == str(commands.IncrementalStyle.SHALLOW)
-                else commands.IncrementalStyle.FINE_GRAINED,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    _check_configuration(configuration)
+    _start_logging_to_directory(configuration.log_directory)
+    start_arguments = command_arguments.StartArguments(
+        changed_files_path=command_argument.changed_files_path,
+        debug=command_argument.debug,
+        enable_memory_profiling=command_argument.enable_memory_profiling,
+        enable_profiling=command_argument.enable_profiling,
+        load_initial_state_from=command_argument.load_initial_state_from,
+        log_identifier=command_argument.log_identifier,
+        logging_sections=command_argument.logging_sections,
+        no_saved_state=command_argument.no_saved_state,
+        no_watchman=no_watchman,
+        noninteractive=command_argument.noninteractive,
+        save_initial_state_to=command_argument.save_initial_state_to,
+        saved_state_project=command_argument.saved_state_project,
+        sequential=command_argument.sequential,
+        show_error_traces=command_argument.show_error_traces,
+        store_type_check_resolution=store_type_check_resolution,
+        terminal=terminal,
+        wait_on_initialization=True,
+    )
+    return v2.restart.run(
+        configuration,
+        command_arguments.IncrementalArguments(
+            output=command_argument.output,
+            no_start=False,
+            start_arguments=start_arguments,
+        ),
+    )
 
 
 @pyre.group(
@@ -1275,7 +1153,8 @@ def servers(context: click.Context) -> int:
     Commands to manipulate multiple Pyre servers.
     """
     if context.invoked_subcommand is None:
-        return _run_servers_list_command(context.obj["arguments"])
+        arguments: command_arguments.CommandArguments = context.obj["arguments"]
+        return v2.servers.run_list(arguments.output)
     # This return value is not used anywhere.
     return ExitCode.SUCCESS
 
@@ -1286,7 +1165,8 @@ def servers_list(context: click.Context) -> int:
     """
     List all running servers.
     """
-    return _run_servers_list_command(context.obj["arguments"])
+    arguments: command_arguments.CommandArguments = context.obj["arguments"]
+    return v2.servers.run_list(arguments.output)
 
 
 @servers.command(name="stop")
@@ -1295,23 +1175,7 @@ def servers_stop(context: click.Context) -> int:
     """
     Stop all running servers.
     """
-    command_argument: command_arguments.CommandArguments = context.obj["arguments"]
-    if command_argument.use_command_v2 is not False:
-        return v2.servers.run_stop()
-    else:
-        configuration = configuration_module.create_configuration(
-            command_argument, Path(".")
-        )
-        return run_pyre_command(
-            commands.Servers(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                subcommand="stop",
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    return v2.servers.run_stop()
 
 
 @pyre.command()
@@ -1334,11 +1198,11 @@ def servers_stop(context: click.Context) -> int:
     "--incremental-style",
     type=click.Choice(
         [
-            str(commands.IncrementalStyle.SHALLOW),
-            str(commands.IncrementalStyle.FINE_GRAINED),
+            str(command_arguments.IncrementalStyle.SHALLOW),
+            str(command_arguments.IncrementalStyle.FINE_GRAINED),
         ]
     ),
-    default=str(commands.IncrementalStyle.FINE_GRAINED),
+    default=str(command_arguments.IncrementalStyle.FINE_GRAINED),
     help="[DEPRECATED] How to approach doing incremental checks.",
 )
 @click.option(
@@ -1361,47 +1225,30 @@ def start(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    if configuration.use_command_v2:
-        _check_configuration(configuration)
-        _start_logging_to_directory(configuration.log_directory)
-        return v2.start.run(
-            configuration,
-            command_arguments.StartArguments(
-                changed_files_path=command_argument.changed_files_path,
-                debug=command_argument.debug,
-                enable_memory_profiling=command_argument.enable_memory_profiling,
-                enable_profiling=command_argument.enable_profiling,
-                load_initial_state_from=command_argument.load_initial_state_from,
-                log_identifier=command_argument.log_identifier,
-                logging_sections=command_argument.logging_sections,
-                no_saved_state=command_argument.no_saved_state,
-                no_watchman=no_watchman,
-                noninteractive=command_argument.noninteractive,
-                save_initial_state_to=command_argument.save_initial_state_to,
-                saved_state_project=command_argument.saved_state_project,
-                sequential=command_argument.sequential,
-                show_error_traces=command_argument.show_error_traces,
-                store_type_check_resolution=store_type_check_resolution,
-                terminal=terminal,
-                wait_on_initialization=wait_on_initialization,
-            ),
-        )
-    else:
-        return run_pyre_command(
-            commands.Start(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                terminal=terminal,
-                store_type_check_resolution=store_type_check_resolution,
-                use_watchman=not no_watchman,
-                incremental_style=commands.IncrementalStyle.SHALLOW
-                if incremental_style == str(commands.IncrementalStyle.SHALLOW)
-                else commands.IncrementalStyle.FINE_GRAINED,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    _check_configuration(configuration)
+    _start_logging_to_directory(configuration.log_directory)
+    return v2.start.run(
+        configuration,
+        command_arguments.StartArguments(
+            changed_files_path=command_argument.changed_files_path,
+            debug=command_argument.debug,
+            enable_memory_profiling=command_argument.enable_memory_profiling,
+            enable_profiling=command_argument.enable_profiling,
+            load_initial_state_from=command_argument.load_initial_state_from,
+            log_identifier=command_argument.log_identifier,
+            logging_sections=command_argument.logging_sections,
+            no_saved_state=command_argument.no_saved_state,
+            no_watchman=no_watchman,
+            noninteractive=command_argument.noninteractive,
+            save_initial_state_to=command_argument.save_initial_state_to,
+            saved_state_project=command_argument.saved_state_project,
+            sequential=command_argument.sequential,
+            show_error_traces=command_argument.show_error_traces,
+            store_type_check_resolution=store_type_check_resolution,
+            terminal=terminal,
+            wait_on_initialization=wait_on_initialization,
+        ),
+    )
 
 
 @pyre.command()
@@ -1430,33 +1277,23 @@ def statistics(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-
-    if configuration.use_command_v2:
-        return v2.statistics.run(
-            configuration,
-            command_arguments.StatisticsArguments(
-                filter_paths=list(filter_paths),
-                log_identifier=command_argument.log_identifier,
-                log_results=log_results,
-                print_aggregates=print_aggregates,
-            ),
-        )
-    else:
-        return run_pyre_command(
-            commands.Statistics(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                filter_paths=list(filter_paths),
-                log_results=log_results,
-                aggregate=print_aggregates,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    return v2.statistics.run(
+        configuration,
+        command_arguments.StatisticsArguments(
+            filter_paths=list(filter_paths),
+            log_identifier=command_argument.log_identifier,
+            log_results=log_results,
+            print_aggregates=print_aggregates,
+        ),
+    )
 
 
 @pyre.command()
+@click.argument(
+    "roots",
+    type=str,
+    nargs=-1,
+)
 @click.option(
     "--working-directory",
     metavar="DIR",
@@ -1468,6 +1305,7 @@ def statistics(
 @click.pass_context
 def coverage(
     context: click.Context,
+    roots: Iterable[str],
     working_directory: str,
 ) -> int:
     """
@@ -1475,19 +1313,7 @@ def coverage(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    if configuration.use_command_v2:
-        return v2.coverage.run(configuration, working_directory)
-    else:
-        return run_pyre_command(
-            commands.Coverage(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                working_directory=working_directory,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    return v2.coverage.run(configuration, working_directory, list(roots))
 
 
 @pyre.command()
@@ -1500,19 +1326,8 @@ def stop(context: click.Context) -> int:
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-    if configuration.use_command_v2:
-        _start_logging_to_directory(configuration.log_directory)
-        return v2.stop.run(configuration)
-    else:
-        return run_pyre_command(
-            commands.Stop(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    _start_logging_to_directory(configuration.log_directory)
+    return v2.stop.run(configuration)
 
 
 @pyre.command()
@@ -1523,20 +1338,8 @@ def validate_models(context: click.Context) -> int:
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-
-    if configuration.use_command_v2:
-        _start_logging_to_directory(configuration.log_directory)
-        return v2.validate_models.run(configuration, output=command_argument.output)
-    else:
-        return run_pyre_command(
-            commands.ValidateModels(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    _start_logging_to_directory(configuration.log_directory)
+    return v2.validate_models.run(configuration, output=command_argument.output)
 
 
 # Need the default argument here since this is our entry point in setup.py
